@@ -29,7 +29,7 @@
 extern "C" {
 #endif
 
-#include "S4R.h"
+#include "Rhelpers.h"
 
 /* Microsoft Visual C++ uses int _getpid()  */
 #ifdef MSVC
@@ -62,10 +62,10 @@ typedef enum enum_dbi_exception {
  * the *_ID macros extract the appropriate scalar.
  */
 
-#define Mgr_Handle s_object
-#define Con_Handle s_object
-#define Res_Handle s_object
-#define Db_Handle  s_object       /* refers to any one of the above */
+#define Mgr_Handle SEXP
+#define Con_Handle SEXP
+#define Res_Handle SEXP
+#define Db_Handle  SEXP       /* refers to any one of the above */
 /* The integer value for the following enum's needs to equal 
  * GET_LENGTH(handle) for the various handles.
  */
@@ -167,32 +167,32 @@ typedef struct st_sdbi_manager {
  * return handles.  All DBI functions (free/get/etc) use the handle 
  * to work with the various dbObjects.
  */
-Mgr_Handle     *RS_DBI_allocManager(const char *drvName, Sint max_con, 
+Mgr_Handle RS_DBI_allocManager(const char *drvName, Sint max_con, 
 				    Sint fetch_default_rec, 
 				    Sint force_realloc);
-void            RS_DBI_freeManager(Mgr_Handle *mgrHandle);
-RS_DBI_manager *RS_DBI_getManager(Db_Handle *handle);
-Mgr_Handle     *RS_DBI_asMgrHandle(Sint pid);   
-s_object       *RS_DBI_managerInfo(Mgr_Handle *mgrHandle);
+void            RS_DBI_freeManager(Mgr_Handle mgrHandle);
+RS_DBI_manager *RS_DBI_getManager(Db_Handle handle);
+Mgr_Handle RS_DBI_asMgrHandle(Sint pid);   
+SEXP RS_DBI_managerInfo(Mgr_Handle mgrHandle);
 
 /* dbConnection */
-Con_Handle        *RS_DBI_allocConnection(Mgr_Handle *mgrHandle, 
+Con_Handle RS_DBI_allocConnection(Mgr_Handle mgrHandle, 
 					  Sint max_res);
-void               RS_DBI_freeConnection(Con_Handle *conHandle);
-RS_DBI_connection *RS_DBI_getConnection(Db_Handle *handle);
-Con_Handle        *RS_DBI_asConHandle(Sint mgrId, Sint conId);
-s_object          *RS_DBI_connectionInfo(Con_Handle *con_Handle);
+void               RS_DBI_freeConnection(Con_Handle conHandle);
+RS_DBI_connection *RS_DBI_getConnection(Db_Handle handle);
+Con_Handle RS_DBI_asConHandle(Sint mgrId, Sint conId);
+SEXP RS_DBI_connectionInfo(Con_Handle con_Handle);
 
 /* dbResultSet */
-Res_Handle        *RS_DBI_allocResultSet(Con_Handle *conHandle);
-void               RS_DBI_freeResultSet(Res_Handle *rsHandle);
-RS_DBI_resultSet  *RS_DBI_getResultSet(Res_Handle *rsHandle);
-Res_Handle        *RS_DBI_asResHandle(Sint pid, Sint conId, Sint resId);
-s_object          *RS_DBI_resultSetInfo(Res_Handle *rsHandle);
+Res_Handle RS_DBI_allocResultSet(Con_Handle conHandle);
+void               RS_DBI_freeResultSet(Res_Handle rsHandle);
+RS_DBI_resultSet  *RS_DBI_getResultSet(Res_Handle rsHandle);
+Res_Handle RS_DBI_asResHandle(Sint pid, Sint conId, Sint resId);
+SEXP RS_DBI_resultSetInfo(Res_Handle rsHandle);
 
 /* utility funs */
-s_object *RS_DBI_validHandle(Db_Handle *handle); /* callable from S/R */
-int       is_validHandle(Db_Handle *handle, HANDLE_TYPE handleType);
+SEXP RS_DBI_validHandle(Db_Handle handle); /* callable from S/R */
+int       is_validHandle(Db_Handle handle, HANDLE_TYPE handleType);
 
 /* a simple object database (mapping table) -- it uses simple linear 
  * search (we don't expect to have more than a handful of simultaneous 
@@ -210,28 +210,30 @@ void  RS_DBI_freeEntry(Sint *table, Sint indx);
 
 /* description of the fields in a result set */
 RS_DBI_fields *RS_DBI_allocFields(int num_fields);
-s_object      *RS_DBI_getFieldDescriptions(RS_DBI_fields *flds);
+SEXP RS_DBI_getFieldDescriptions(RS_DBI_fields *flds);
 void           RS_DBI_freeFields(RS_DBI_fields *flds);
 
 /* we (re)allocate the actual output list in here (with the help of
  * RS_DBI_fields).  This should be some kind of R/S "relation"
  * table, not a dataframe nor a list.
  */
-void  RS_DBI_allocOutput(s_object *output, 
+void  RS_DBI_allocOutput(SEXP output, 
 			RS_DBI_fields *flds,
 			Sint num_rec,
 			Sint expand);
-void RS_DBI_makeDataFrame(s_object *data);
+void RS_DBI_makeDataFrame(SEXP data);
 
 /* TODO: We need to elevate RS_DBI_errorMessage to either
  * dbManager and/or dbConnection methods.  I still need to 
  * go back and re-code the error-handling throughout, darn!
  */
 void  RS_DBI_errorMessage(char *msg, DBI_EXCEPTION exceptionType);
-void  RS_DBI_setException(Db_Handle *handle, 
+void  RS_DBI_setException(Db_Handle handle, 
 			  DBI_EXCEPTION exceptionType,
 			  int errorNum, 
 			  const char *errorMsg);
+void DBI_MSG(char *msg, DBI_EXCEPTION exception_type, char *driver);
+
 /* utility funs (copy strings, convert from R/S types to string, etc.*/
 char     *RS_DBI_copyString(const char *str);
 char     *RS_DBI_nCopyString(const char *str, size_t len, int del_blanks);
@@ -248,13 +250,13 @@ struct data_types {
 /* return the primitive type name for a primitive type id */
 char     *RS_DBI_getTypeName(Sint typeCode, const struct data_types table[]);
 /* same, but callable from S/R and vectorized */
-s_object *RS_DBI_SclassNames(s_object *types);  
+SEXP RS_DBI_SclassNames(SEXP types);  
 
-s_object *RS_DBI_createNamedList(char  **names, 
+SEXP RS_DBI_createNamedList(char  **names, 
 				 Stype *types,
 				 Sint  *lengths,
 				 Sint  n);
-s_object *RS_DBI_copyFields(RS_DBI_fields *flds);
+SEXP RS_DBI_copyFields(RS_DBI_fields *flds);
 
 void RS_na_set(void *ptr, Stype type);
 int  RS_is_na(void *ptr, Stype type);
